@@ -4,6 +4,7 @@ use strict;
 use warnings;
 no warnings 'uninitialized';
 
+use Yote;
 use Yote::Server;
 
 use CGI;
@@ -20,7 +21,7 @@ unless( $main::yote_server ) {
     my $options = Yote::Server::load_options( $yote_root_dir );
 
     $main::yote_server = new Yote::Server( $options );
-    $main::yote_server->ensure_locker;
+#    $main::yote_server->ensure_locker;
 
 }
 my $cgi = CGI->new;
@@ -32,7 +33,13 @@ eval {
     $out_json = $main::yote_server->invoke_payload( $json_payload );
 };
 
+if( ref $@ eq 'HASH' ) {
+    $out_json = to_json( $@ );
+    undef $@;
+    print STDERR Data::Dumper->Dump(["GOTTY IN CGI",$out_json,$@]);
+}
 if( $@ ) {
+    print STDERR Data::Dumper->Dump([$@,"ERRORGOT"]);
     print $cgi->header( -status => '400 BAD REQUEST' );
 } else {
     print $cgi->header(
