@@ -17,7 +17,7 @@ use URI::Escape;
 
 use vars qw($VERSION);
 
-$VERSION = '1.05';
+$VERSION = '1.06';
 
 our $DEBUG = 1;
 
@@ -236,32 +236,6 @@ sub _log {
     print STDERR "Yote::Server : $msg\n" if $sev <= $DEBUG;
 }
 
-sub __transform_params {
-    #
-    # Recursively transforms incoming parameters into values, yote objects, or non yote containers.
-    # This checks to make sure that the parameters are allowed by the given token.
-    # Throws execptions if the parametsr are not allowed, or if a reference that is not a hash or array
-    # is encountered.
-    #
-    my( $self, $param, $token, $server_root ) = @_;
-
-    if( ref( $param ) eq 'HASH' ) {
-        return { map { $_ => $self->__transform_params($param->{$_}, $token, $server_root) } keys %$param };
-    } 
-    elsif( ref( $param ) eq 'ARRAY' ) {
-        return [ map { $self->__transform_params($_, $token, $server_root) } @$param ];
-    } elsif( ref( $param ) ) {
-        die "Transforming Params: got weird ref '" . ref( $param ) . "'";
-    }
-    if( index( $param, 'v' ) == 0 ) {
-        if( $server_root->_getMay( $param, $token ) ) {
-            return $self->{STORE}->_xform_out( $param );
-        }
-        die( "Bad Req Param, server says no : $param" );
-    }
-    return $self->{STORE}->fetch($param); #oops!
-} #__transform_params
-
 sub _find_ids_in_data {
     my $data = shift;
     my $r = ref( $data );
@@ -276,6 +250,47 @@ sub _find_ids_in_data {
     }
 } #_find_ids_in_data
 
+# EXPERIMETNAL - this will return the entire public tree. The idea is to program
+# without having to explicitly shove data across. This errs on the side of much
+# more data, so relies on private data and method calls (encapsulation) to
+# mitigate this
+
+sub _unroll_ids {
+    my( $store, $ids, $seen ) = @_;
+    $seen  //= {};
+
+    my( @items ) = ( map { $store->fetch($_) } @$ids );
+
+#    print STDERR Data::Dumper->Dump(["INITIAL IDS -->" . join(",",@$ids),"IDS FROM THOSE -->" . join(",",map { $store->_get_id($_) } @items ),"SEEN --> " .join(",",keys %$seen),"UNROLL"]);
+    
+    my @outids;
+    for my $item( @items ) {
+        my $iid = $store->_get_id($item);
+        my $r = ref( $item );
+#        print STDERR Data::Dumper->Dump(["  CHECKING ($iid) ($r)"]);
+        $seen->{$iid}++;
+        if( $r eq 'ARRAY' ) {
+            push @outids, grep { ! $seen->{$_}++ } map { $store->_get_id($_)  } grep { ref($_) } @$item;
+        }
+        elsif( $r eq 'HASH' ) {
+            push @outids, grep { ! $seen->{$_}++ } map { $store->_get_id($_)  } grep { ref($_) } values %$item;
+        }
+        else {
+            my $data = $item->{DATA};
+#            print STDERR Data::Dumper->Dump([$data, "DATA for $item->{ID}"]);
+            push @outids, map { $data->{$_} } grep { /^[^_]/ && $data->{$_} != /^v/ && ! $seen->{$data->{$_}}++ } keys %$data;
+        }
+    }
+#    print STDERR Data::Dumper->Dump(["SEEN --> " .join(",",keys %$seen),"OUTY --> " .join(",",@outids),"SEENY?"]);
+    
+    _unroll_ids( $store, \@outids, $seen ) if @outids;
+
+
+#    print STDERR Data::Dumper->Dump(["SEEN --> " .join(",",keys %$seen),"SEEN NOWUNROLL!"]);
+    
+    [ keys %$seen ];
+} #_unroll_ids
+
 sub _process_request {
     #
     # Reads incomming request from the socket, parses it, performs it and
@@ -375,7 +390,11 @@ sub _process_request {
         eval {
             $out_json = $self->invoke_payload( $data );
         };
-        if( $@ ) {
+
+        if( ref $@ eq 'HASH' ) {
+            $out_json = to_json( $@ );
+        } 
+        elsif( $@ ) {
             _log( $@ );
             $sock->print( "HTTP/1.1 400 BAD REQUEST\n\n" );
             $sock->close;
@@ -393,7 +412,7 @@ sub _process_request {
         $sock->print( "HTTP/1.1 200 OK\n" . join ("\n", @headers). "\n\n$out_json\n" );
         
         $sock->close;
-        $self->{STORE}->stow_all;
+
         exit;
 
     } #child
@@ -407,21 +426,29 @@ sub invoke_payload {
     my $req_data = from_json( shift );
 
     my( $obj_id, $token, $action, $params ) = @$req_data{ 'i', 't', 'a', 'pl' };
-
-    _log( "\n   (params [$$])--> : ".Data::Dumper->Dump([$params]) );
+#    print STDERR Data::Dumper->Dump(["'$action' on '$obj_id' with token [$token]) and params --> : ".join( ",", @$params )]);
     my $server_root = $self->{STORE}->fetch_server_root;
     
     my $server_root_id = $server_root->{ID};
-    my $session = $server_root->_fetch_session( $token );
+    my $session = $token && $token ne '_' ? $server_root->_fetch_session( $token ) : undef;
 
+#    print STDERR Data::Dumper->Dump(["TOKEN [$token] --> Session ($session)"]);
+
+    if( $session ) {
+#        print STDERR Data::Dumper->Dump([$session->{DATA},"SESSDA"]);
+    }
+    
+    my $ids2times;
+    if( $session ) {
+        $ids2times = $session->get__has_ids2times;
+#        print STDERR Data::Dumper->Dump(["SESSION ($session) HAS --> " . join(',',sort {$a <=>$b }keys %$ids2times)]);
+    }
+    
     unless( $obj_id eq '_' || 
-            $obj_id eq $server_root_id || 
-            ( $obj_id > 0 && 
-              $session && 
-              $server_root->_getMay( $obj_id, $session->get__token ) ) ) {
-        
+            $obj_id eq $server_root_id ||
+            ( $ids2times->{$obj_id} ) ) {
         # tried to do an action on an object it wasn't handed. do a 404
-        die( "Bad Path" );
+        die( "client with token [$token] and session ($session) tried to invoke on obj id '$obj_id' which it does not have" );
     }
     if( substr( $action, 0, 1 ) eq '_' ) {
         die( "Private method called" );
@@ -431,20 +458,20 @@ sub invoke_payload {
         die( "Bad Req Param Not Array : $params" );
     }
 
+    my $store = $self->{STORE};
+
     # now things are getting a bit more complicated. The params passed in
     # are always a list, but they may contain other containers that are not
     # yote objects. So, transform the incomming parameter list and check all
     # yote objects inside for may. Use a recursive helper function for this.
+    my $in_params = $store->__transform_params( $params, $session );
 
-    my $in_params = $self->__transform_params( $params, $token, $server_root );
-
-    my $store = $self->{STORE};
-
+    #
+    # This obj is the object that the method call is on
+    #
     my $obj = $obj_id eq '_' ? $server_root :
         $store->fetch( $obj_id );
 
-
-
     unless( $obj->can( $action ) ) {
         die( "Bad Req : invalid method :'$action'" );
     }
@@ -454,70 +481,115 @@ sub invoke_payload {
         $obj->{SESSION}{SERVER_ROOT} = $server_root;
     }
 
+    # <<------------- the actual method call --------------->>
     my(@res) = ($obj->$action( @$in_params ));
-    delete $obj->{SESSION};
-        
+#    print STDERR Data::Dumper->Dump([\@res,"MYREZ"]);
+    # this is included in what is  returned to the client 
     my $out_res = $store->_xform_in( \@res, 'allow datastructures' );
 
-    my @has = _find_ids_in_data( $out_res );
-    my @mays = @has, $store->_find_ids_referenced( @has );
 
-    my $ids_to_update;
-    if ( ( $action eq 'fetch_root' || $action eq 'init_root' || $action eq 'fetch_app' )  && ( $obj_id eq '_' || $obj_id eq $server_root_id ) ) {
-        # if there is a token, make it known that the token 
-        # has received server root data
-        $ids_to_update = [ $server_root_id, grep { $_ ne $server_root_id } @has ];
-        if ( $token  ) {
-            push @has, $server_root_id;
+    # the ids that were referenced explicitly in the
+    # method call.
+    my @out_ids = _find_ids_in_data( $out_res );
+#    print STDERR Data::Dumper->Dump(["RESULTS [$token] ($session) --> " . join(',',@res),"RESSU"]);
+#    print STDERR Data::Dumper->Dump(["OUT IDS [$token] ($session) --> ". join(",",@out_ids)]);
+
+    #
+    # Based on the return value of the method call,
+    #   these ids are ones that the client should have.
+    #   We will check to see if these need updates
+    #
+    my @should_have = ( @{ _unroll_ids( $store, \@out_ids ) } );
+#    print STDERR Data::Dumper->Dump(["SHOULDS [$token] ($session) --> " . join(",",@should_have)]);
+
+    my( @updates, %methods );
+
+    #
+    # fetch root always sends the root back for an update
+    #
+    if ( ( $action eq 'fetch_root' || $action eq 'init_root' || $action eq 'fetch_app' )
+         && ( $obj_id eq '_' || $obj_id eq $server_root_id ) ) {
+        push @should_have, $server_root_id;
+        my $cls = ref( $server_root );
+        my $d = $server_root->{DATA};
+        push @updates, {
+            id    => $server_root_id,
+            cls   => $cls,
+            data  => { map { $_ => $d->{$_} } grep { index($_,"_") != 0 } keys %$d },
+        };
+        if( $action eq 'fetch_app' ) {
+            my( $app, $login ) = @res;
+            $d = $app->{DATA};
+            my $ra = ref( $app );
+            push @updates, {
+                id    => $app->{ID},
+                cls   => $ra,
+                data  => { map { $_ => $d->{$_} } grep { index($_,"_") != 0 } keys %$d },
+            };
+            $methods{ $ra } = $app->_callable_methods;
+            if( $session ) {
+                $ids2times->{$app->{ID}} = [Time::HiRes::gettimeofday];
+            }
+            my $lr = ref $login;
+            if( $lr ) {
+                $d = $login->{DATA};
+                push @updates, {
+                    id    => $login->{ID},
+                    cls   => $lr,
+                    data  => { map { $_ => $d->{$_} } grep { index($_,"_") != 0 } keys %$d },
+                };
+                $methods{ $lr } = $login->_callable_methods;
+                if( $session ) {
+                    $ids2times->{$login->{ID}} = [Time::HiRes::gettimeofday];
+                }
+            }
         }
-    } else {
-        $ids_to_update = $server_root->_updates_needed( $token, \@has );
+        $methods{ $cls } = $server_root->_callable_methods;
+    } 
+    if( $session)  {
+
+        #
+        # check if existing are in the session
+        #
+        for my $should_have_id ( @should_have, keys %$ids2times ) {
+            # check if this needs an update
+            my( $client_s, $client_ms )  = @{ $ids2times->{$should_have_id} || [] };
+            my( $server_s, $server_ms )  = $store->_last_updated( $should_have_id );
+#            print STDERR Data::Dumper->Dump(["Check ($should_have_id) --> c($client_s,$client_ms) <=> s($server_s,$server_ms) ". ( ( $client_s == 0 || $server_s > $client_s || ($server_s == $client_s && $server_ms > $client_ms )) ? " INCLUDE " : " DONT BOTHER ")]);
+            if( $client_s == 0 || $server_s > $client_s || ($server_s == $client_s && $server_ms > $client_ms )) {
+                my $should_have_obj = $store->fetch( $should_have_id );
+                my $ref = ref( $should_have_obj );
+                my $data;
+                if( $ref eq 'ARRAY' ) {
+                    $data = [ map { $store->_xform_in( $_ ) } @$should_have_obj ];
+                } elsif(  $ref eq 'HASH' ) {
+                    $data = { map { $_ =>  $store->_xform_in( $should_have_obj->{$_} ) } keys %$should_have_obj };
+                } else {
+                    my $d = $should_have_obj->{DATA};
+                    $data = { map { $_ => $d->{$_} } grep { index($_,"_") != 0 } keys %$d },
+                    $methods{$ref} ||= $should_have_obj->_callable_methods;
+                }
+                my $update = {
+                    id    => $should_have_id,
+                    cls   => $ref,
+                    data  => $data,
+                };
+                push @updates, $update;
+                $ids2times->{$should_have_id} = [Time::HiRes::gettimeofday];
+#                print STDERR Data::Dumper->Dump(["ADDING '$should_have_id' to ids2times"]);
+            }
+        } #each object the client should have
+#        print STDERR Data::Dumper->Dump(["TOKEN [$token] SESSION ($session) NOW HAS {".$self->{STORE}->_get_id($ids2times)."} --> " . join(',',sort {$a <=>$b }keys %$ids2times)]);
     }
     
-    my( @updates, %methods );
-    for my $obj_id (@$ids_to_update) {
-        my $obj = $store->fetch( $obj_id );
-        my $ref = ref( $obj );
-        
-        my( $data );
-        if ( $ref eq 'ARRAY' ) {
-            $data = [ 
-                map { my $d = $store->_xform_in( $_ );
-                      push @mays, $d;
-                      $d } 
-                @$obj ];
-        } elsif ( $ref eq 'HASH' ) {
-            $data = {
-                map { my $d = $store->_xform_in( $obj->{$_} );
-                      push @mays, $d;
-                      $_ => $d }
-                keys %$obj };
-        } else {
-            my $obj_data = $obj->{DATA};
-            
-            $data = {
-                map { my $d = $obj_data->{$_};
-                      push @mays, $d;
-                      $_ => $d }
-                grep { $_ !~ /^_/ }
-                keys %$obj_data };
-            $methods{$ref} ||= $obj->_callable_methods;
-        }
-        push @has, $obj_id;
-        my $update = {
-            id    => $obj_id,
-            cls   => $ref,
-            data  => $data,
-        };
-        push @updates, $update;
-    } #each obj_id to update
-
-    $server_root->_setHasAndMay( \@has, \@mays, $token );
-
     my $out_json = to_json( { result  => $out_res,
                               updates => \@updates,
                               methods => \%methods,
                             } );
+    delete $obj->{SESSION};
+#    print STDERR Data::Dumper->Dump([$out_json,"OUTTTY"]);
+    $self->{STORE}->stow_all;
+
     return $out_json;
 } #invoke_payload
 
@@ -537,8 +609,11 @@ sub _new { #Yote::ServerStore
     # keeps track of when any object had been last updated.
     # use like $self->{OBJ_UPDATE_DB}->put_record( $obj_id, [ time ] );
     # or my( $time ) = @{ $self->{OBJ_UPDATE_DB}->get_record( $obj_id ) };
-    $self->{OBJ_UPDATE_DB} = Data::RecordStore::FixedStore->open( "L", "$args->{root}/OBJ_META" );
-    $self->{OBJ_UPDATE_DB}->put_record( $self->{ID}, [ Time::HiRes::time ] );
+    $self->{OBJ_UPDATE_DB} = Data::RecordStore::FixedStore->open( "LL", "$args->{root}/OBJ_META" );
+
+    my( $m, $ms ) = ( Time::HiRes::gettimeofday  );
+    $self->{OBJ_UPDATE_DB}->put_record( $self->{ID}, [ $m, $ms ] );
+
     $self;
 } #_new
 
@@ -546,7 +621,9 @@ sub _dirty {
     my( $self, $ref, $id ) = @_;
     $self->SUPER::_dirty( $ref, $id );
     $self->{OBJ_UPDATE_DB}->ensure_entry_count( $id );
-    $self->{OBJ_UPDATE_DB}->put_record( $id, [ Time::HiRes::time ] );
+
+    my( $m, $ms ) = ( Time::HiRes::gettimeofday  );
+    $self->{OBJ_UPDATE_DB}->put_record( $id, [ $m, $ms ] );
 }
 
 sub stow_all {
@@ -554,29 +631,55 @@ sub stow_all {
     for my $obj (values %{$self->{_DIRTY}} ) {
         my $obj_id = $self->_get_id( $obj );
         $self->{OBJ_UPDATE_DB}->ensure_entry_count( $obj_id );
-        $self->{OBJ_UPDATE_DB}->put_record( $obj_id, [ Time::HiRes::time ] );
     }
     $self->SUPER::stow_all;
 } #stow_all
 
 sub _stow {
-    my( $self, $obj ) = @_;
+    my( $self, $obj, $no_timesave ) = @_;
     $self->SUPER::_stow( $obj );
 
-    my $obj_id = $self->_get_id( $obj );
-    $self->{OBJ_UPDATE_DB}->put_record( $obj_id, [ Time::HiRes::time ] );
+    unless( $no_timesave ) {
+        my $obj_id = $self->_get_id( $obj );
+    }
 }
 
 sub _last_updated {
     my( $self, $obj_id ) = @_;
-    my( $time ) = @{ $self->{OBJ_UPDATE_DB}->get_record( $obj_id ) };
-    $time;
+    my( $s, $ms ) = @{ $self->{OBJ_UPDATE_DB}->get_record( $obj_id ) };
+    $s, $ms;
 }
 
 sub _log {
     Yote::Server::_log(shift);
 }
 
+
+sub __transform_params {
+    #
+    # Recursively transforms incoming parameters into values, yote objects, or non yote containers.
+    # This checks to make sure that the parameters are allowed by the given token.
+    # Throws execptions if the parametsr are not allowed, or if a reference that is not a hash or array
+    # is encountered.
+    #
+    my( $self, $param, $session ) = @_;
+
+    if( ref( $param ) eq 'HASH' ) {
+        return { map { $_ => $self->__transform_params($param->{$_}, $session) } keys %$param };
+    } 
+    elsif( ref( $param ) eq 'ARRAY' ) {
+        return [ map { $self->__transform_params($_, $session) } @$param ];
+    } elsif( ref( $param ) ) {
+        die "Transforming Params: got weird ref '" . ref( $param ) . "'";
+    }
+    if( index( $param, 'v' ) != 0 && !$session->get__has_ids2times({})->{$param} ) {
+        # obj id given, but the client should not have that id
+        die "Client requested obj with id '$param' which it should not have.";
+    }
+    return $self->_xform_out( $param );
+} #__transform_params
+
+
 #
 # Unlike the superclass version of this, this provides an arguemnt to
 # allow non-yote datastructures to be returned. The contents of those
@@ -589,7 +692,7 @@ sub _xform_in {
     if( $r ) {
         if( $allow_datastructures) {
             # check if this is a yote object
-            if( ref( $val ) eq 'ARRAY' && ! tied @$val ) {
+            if( ref( $val ) eq 'ARRAY' && ! tied( @$val ) ) {
                 return [ map { ref $_ ? $self->_xform_in( $_, $allow_datastructures ) : "v$_" } @$val ];
             }
             elsif( ref( $val ) eq 'HASH' && ! tied %$val ) {
@@ -671,6 +774,11 @@ sub _log {
     Yote::Server::_log(shift);
 }
 
+sub _err {
+    shift; #self
+    die { err => shift };
+}
+
 $Yote::ServerObj::PKG2METHS = {};
 sub __discover_methods {
     my $pkg = shift;
@@ -682,7 +790,7 @@ sub __discover_methods {
     no strict 'refs';
     my @m = grep { $_ !~ /::/ } keys %{"${pkg}\::"};
     if( $pkg eq 'Yote::ServerObj' ) { #the base, presumably
-        return [ grep { $_ !~ /^(_|[gs]et_|(can|[sg]et|VERSION|AUTOLOAD|DESTROY|CARP_TRACE|BEGIN|isa|PKG2METHS|ISA)$)/ } @m ];
+        return [ sort grep { $_ !~ /^(_|[gs]et_|(can|[sg]et|VERSION|AUTOLOAD|DESTROY|CARP_TRACE|BEGIN|isa|import|PKG2METHS|ISA)$)/ } @m ];
     }
 
     my %hasm = map { $_ => 1 } @m;
@@ -694,7 +802,7 @@ sub __discover_methods {
     
     my $base_meths = __discover_methods( 'Yote::ServerObj' );
     my( %base ) = map { $_ => 1 } 'AUTOLOAD', @$base_meths;
-    $meths = [ grep { $_ !~ /^(_|[gs]et_|(can|[sg]et|VERSION|AUTOLOAD|DESTROY|BEGIN|isa|PKG2METHS|ISA)$)/ && ! $base{$_} } @m ];
+    $meths = [ sort grep { $_ !~ /^(_|[gs]et_|(can|[sg]et|VERSION|AUTOLOAD|DESTROY|BEGIN|isa|import|PKG2METHS|ISA)$)/ && ! $base{$_} } @m ];
     $Yote::ServerObj::PKG2METHS->{$pkg} = $meths;
     
     $meths;
@@ -709,14 +817,6 @@ sub _callable_methods {
     __discover_methods( $pkg );
 } # _callable_methods
 
-sub get {
-    my( $self, $fld, $default ) = @_;
-    if( index( $fld, '_' ) == 0 ) {
-        die "Cannot get private field $fld";
-    }
-    $self->_get( $fld, $default );
-} #get
-
 
 sub _get {
     my( $self, $fld, $default ) = @_;
@@ -731,19 +831,6 @@ sub _get {
 } #_get
 
 
-sub set {
-    my( $self, $fld, $val ) = @_;
-    if( index( $fld, '_' ) == 0 ) {
-        die "Cannot set private field";
-    }
-    my $inval = $self->{STORE}->_xform_in( $val );
-    $self->{STORE}->_dirty( $self, $self->{ID} ) if $self->{DATA}{$fld} ne $inval;
-    $self->{DATA}{$fld} = $inval;
-    
-    $self->{STORE}->_xform_out( $self->{DATA}{$fld} );
-} #set
-
-
 # ------- END Yote::ServerObj
 
 package Yote::ServerRoot;
@@ -753,7 +840,6 @@ use base 'Yote::ServerObj';
 sub _init {
     my $self = shift;
     $self->set__doesHave_Token2objs({});
-    $self->set__mayHave_Token2objs({});
     $self->set__apps({});
     $self->set__token_timeslots([]);
     $self->set__token_timeslots_metadata([]);
@@ -766,96 +852,27 @@ sub _log {
 
 sub _fetch_session {
     my( $self, $token ) = @_;
+    
+    $self->{STORE}->lock( 'token_mutex' );
     my $slots = $self->get__token_timeslots();
+#    print STDERR Data::Dumper->Dump(["FETCH SESSION GOT SLOTS (".$self->{STORE}->_get_id($slots).") for token [$token]"]);
     for( my $i=0; $i<@$slots; $i++ ) {
+#        print STDERR Data::Dumper->Dump(["   SLOT (".$self->{STORE}->_get_id($slots->[$i]).") has <".join(",",keys %{$slots->[$i]{$token}}).">"]);
         if( my $session = $slots->[$i]{$token} ) {
-            if( $i < $#$slots ) {
+            #if( $i < $#$slots ) {
+            if( $i > 0 ) {
                 # make sure this is in the most current 'boat'
                 $slots->[0]{ $token } = $session;
             }
+#            print STDERR Data::Dumper->Dump(["FOUND SESSION ($session)"]);
+            $self->{STORE}->unlock( 'token_mutex' );
             return $session;
         }
     }
-    
+    $self->{STORE}->unlock( 'token_mutex' );
+    0;
 } #_fetch_sesion
 
-
-sub _resetHasAndMay {
-    my( $self, $tokens, $doesHaveOnly ) = @_;
-
-    $self->{STORE}->lock( "_has_and_may_Token2objs" );
-    my( @lists ) = $doesHaveOnly ? qw( doesHave ) : qw( doesHave mayHave );
-    for ( @lists ) {
-        my $item = "_${_}_Token2objs";
-        my $token2objs = $self->_get( $item );
-        for my $token (@$tokens) {
-            delete $token2objs->{ $token };
-        }
-    }
-    $self->{STORE}->unlock( "_has_and_may_Token2objs" );
-
-} #_resetHasAndMay
-
-sub _setHasAndMay {
-    my( $self, $has, $may, $token ) = @_;
-
-    $self->{STORE}->lock( "_has_and_may_Token2objs" );
-
-    # has
-    my $obj_data = $self->get__doesHave_Token2objs;
-    for my $id (@$has) {
-        next if index( $id, 'v' ) == 0 || $token eq '_';
-        $obj_data->{$token}{$id} = Time::HiRes::time;
-    }
-    $self->{STORE}->_stow( $obj_data );
-    
-    # may
-    $obj_data = $self->get__mayHave_Token2objs;
-    for my $id (@$may) {
-        next if index( $id, 'v' ) == 0 || $token eq '_';
-        $obj_data->{$token}{$id} = Time::HiRes::time;
-    }
-    $self->{STORE}->_stow( $obj_data );
-    
-    $self->{STORE}->unlock( "_has_and_may_Token2objs" );
-} #_setHasAndMay
-
-
-sub _getMay {
-    my( $self, $id, $token ) = @_;
-    return 1 if index( $id, 'v' ) == 0;
-    return 0 if $token eq '_';
-    my $obj_data = $self->get__mayHave_Token2objs;
-    $obj_data->{$token} && $obj_data->{$token}{$id};
-}
-
-
-
-sub _updates_needed {
-    my( $self, $token, $outRes ) = @_;
-    return [] if $token eq '_';
-
-    my $obj_data = $self->get__doesHave_Token2objs()->{$token};
-    my $store = $self->{STORE};
-    my( @updates );
-    for my $obj_id (@$outRes, keys %$obj_data ) {
-        next if index( $obj_id, 'v' ) == 0;
-        my $last_update_sent = $obj_data->{$obj_id};
-        my $last_updated = $store->_last_updated( $obj_id );
-        if( $last_update_sent <= $last_updated || $last_updated == 0 ) {
-            unless( $last_updated ) {
-                $store->{OBJ_UPDATE_DB}->put_record( $obj_id, [ Time::HiRes::time ] );
-            }
-            push @updates, $obj_id;
-        }
-    }
-    \@updates;
-} #_updates_needed
-
-sub create_token {
-    shift->_create_session->get__token;
-}
-
 sub _create_session {
     my $self = shift;
     my $tries = shift;
@@ -875,7 +892,6 @@ sub _create_session {
 
     $self->{STORE}->lock( 'token_mutex' );
 
-
     #
     # A list of slot 'boats' which store token -> ip
     #
@@ -894,12 +910,13 @@ sub _create_session {
     for( my $i=0; $i<@$slot_data; $i++ ) {
         return $self->_create_session( $tries++ ) if $slots->[ $i ]{ $token };
     }
-
+#    print STDERR Data::Dumper->Dump(["NEW TOKEN {{{$token}}"]);
     #
     # See if the most recent time slot is current. If it is behind, create a new current slot
     # create a new most recent boat.
     #
-    my $session = $self->{STORE}->newobj( { 
+    my $session = $self->{STORE}->newobj( {
+        _has_ids2times => {},
         _token => $token } );
     if( $slot_data->[ 0 ] == $current_time_chunk ) {
         $slots->[ 0 ]{ $token } = $session;
@@ -908,7 +925,6 @@ sub _create_session {
         unshift @$slots, { $token => $session };
     }
     
-
     #
     # remove this token from old boats so it doesn't get purged
     # when in a valid boat.
@@ -917,24 +933,8 @@ sub _create_session {
         delete $slots->[$i]{ $token };
     }
 
-    #
-    # Purge tokens in expired boats.
-    #
-    my @to_remove;
-    while( @$slot_data ) {
-        if( $slot_data->[$#$slot_data] < $earliest_valid_time_chunk ) {
-            pop @$slot_data;
-            my $old = pop @$slots;
-            push @to_remove, keys %$old;
-        } else {
-            last;
-        }
-    }
-
-    if( @to_remove ) {
-        $self->_resetHasAndMay( \@to_remove );
-    }
-    
+    $self->{STORE}->_stow( $slots );
+    $self->{STORE}->_stow( $slot_data );
     $self->{STORE}->unlock( 'token_mutex' );
     
     return $session;
@@ -943,16 +943,24 @@ sub _create_session {
 
 sub _destroy_session {
     my( $self, $token ) = @_;
+    
+    $self->{STORE}->lock( 'token_mutex' );
     my $slots = $self->get__token_timeslots();
     for( my $i=0; $i<@$slots; $i++ ) {
         delete $slots->[$i]{ $token };
     }
-    print STDERR Data::Dumper->Dump(["DESTROY '$token'",$slots]);
-
-    $self->_resetHasAndMay( [$token] );
+    $self->{STORE}->_stow( $slots );
+    $self->{STORE}->unlock( 'token_mutex' );
     1;
 } #_destroy_session
 
+#
+# Needed for when no logins are going to happen
+#
+sub create_token {
+    shift->_create_session->get__token;
+}
+
 #
 # Returns the app and possibly a logged in account
 #
@@ -984,26 +992,9 @@ sub init_root {
     my $self = shift;
     my $session = $self->{SESSION} || $self->_create_session;
     my $token = $session->get__token;
-    $self->_resetHasAndMay( [ $token ], 'doesHaveOnly' );
     return $self, $token;
 }
 
-sub fetch {
-    my( $self, @ids ) = @_;
-    
-    $self->{SESSION} && ( my $token = $self->{SESSION}->get__token ) || return;
-    
-    my $mays = $self->get__mayHave_Token2objs;
-    my $may = $self->get__mayHave_Token2objs()->{$token};
-    my $store = $self->{STORE};
-
-    my @ret = map { $store->fetch($_) }
-      grep { ! ref($_) && $may->{$_}  }
-    @ids;
-    die "Invalid id(s) ".join(",",grep { !ref($_) && !$may->{$_} } @ids) unless @ret == @ids;
-    @ret;
-} #fetch
-
 # while this is a non-op, it will cause any updated contents to be 
 # transfered to the caller automatically
 sub update {
